﻿--[[
    
]]
local tuf_unitframe = TuckUF.unitframe
local tuf_controlframe = TuckUF.controlframe
local tuf_network = TuckUF.network

tuf_controlframe.timer = 0
tuf_controlframe.elapsed = 0

tuf_controlframe.assist_units = {"party1", "party2", "party3", "party4", "pet"}

--[[
    List of all unit frames
]]
tuf_controlframe.unitframes = {}

--[[
    Libraries
]]
local IFrameFactory = IFrameFactory("1.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local TRACE = function(msg) ChatFrame1:AddMessage(msg) end

--[[
    Library interface: IFrameManeger
]]
local ifm_iface = IFrameManager:Interface()
function ifm_iface:getName(frame)
    return frame.unit
end

local ifm_raid_iface = IFrameManager:Interface()
function ifm_raid_iface:getName(frame)
    return "raid "..frame.group
end
function ifm_raid_iface:getBorder(frame)
    return 0, 0, 0, 0
end

--[[
    List of all created unit frames
    name = config
    config.unit: unit id
    config.config: configuration id (see tuf_options.lua)
    config.frame_type: "unit" or "raidgroup"
    config.manual_update: if true, the frame is update in the OnUpdate function
    config.default_layout: default layout
]]

local unit_frames =
{
    player = {unit = "player", config = "player", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPLEFT", "UIParent", "TOPLEFT", 60, -20}},
    pet = {unit = "pet", config = "pet", frame_type ="unit", manual_update = false, 
        default_layout = {"TOPLEFT", "tufFrame_player", "BOTTOMLEFT", 0, -24}},
    target = {unit = "target", config = "target", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPLEFT", "tufFrame_player", "TOPRIGHT", 100, 0}},  
    targettarget = {unit = "targettarget", config = "tot", frame_type = "unit", manual_update = true, 
        default_layout = {"TOPLEFT", "tufFrame_target", "TOPRIGHT", 2, 0}}, 
    targettargettarget = {unit = "targettargettarget", config = "totot", frame_type = "unit", manual_update = true, 
        default_layout = {"TOPLEFT", "tufFrame_targettarget", "BOTTOMLEFT", 0, 4}},  
    focus = {unit = "focus", config = "focus", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPLEFT", "tufFrame_target", "TOPRIGHT", 150, 0}}, 
    focustarget = {unit = "focustarget", config = "tof", frame_type = "unit", manual_update = true, 
        default_layout = {"TOPLEFT", "tufFrame_focus", "TOPRIGHT", 2, 0}},  
    focustargettarget = {unit = "focustargettarget", config = "totof", frame_type = "unit", manual_update = true, 
        default_layout = {"TOPLEFT", "tufFrame_focustarget", "BOTTOMLEFT", 0, 4}}, 
    party1 = {unit = "party1", config = "party", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPLEFT", "tufFrame_pet", "BOTTOMLEFT", 0, -12}},  
    party2 = {unit = "party2", config = "party", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPLEFT", "tufFrame_party1", "BOTTOMLEFT", 0, -24}}, 
    party3 = {unit = "party3", config = "party", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPLEFT", "tufFrame_party2", "BOTTOMLEFT", 0, -24}},  
    party4 = {unit = "party4", config = "party", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPLEFT", "tufFrame_party3", "BOTTOMLEFT", 0, -24}},  
    partypet1 = {unit = "partypet1", config = "party_pet", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPRIGHT", "tufFrame_party1", "BOTTOMRIGHT", 0, 0}},  
    partypet2 = {unit = "partypet2", config = "party_pet", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPRIGHT", "tufFrame_party2", "BOTTOMRIGHT", 0, 0}},  
    partypet3 = {unit = "partypet3", config = "party_pet", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPRIGHT", "tufFrame_party3", "BOTTOMRIGHT", 0, 0}},  
    partypet4 = {unit = "partypet4", config = "party_pet", frame_type = "unit", manual_update = false, 
        default_layout = {"TOPRIGHT", "tufFrame_party4", "BOTTOMRIGHT", 0, 0}}, 
    raidgroup1 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 1, 
        default_layout = {"TOPLEFT", "tufFrame_party4", "BOTTOMLEFT", 0, -40}},  
    raidgroup2 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 2, 
        default_layout = {"TOPLEFT", "tufFrame_raidgroup1", "TOPRIGHT", 0, 0}},  
    raidgroup3 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 3, 
        default_layout = {"TOPLEFT", "tufFrame_raidgroup2", "TOPRIGHT", 0, 0}},  
    raidgroup4 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 4, 
        default_layout = {"TOPLEFT", "tufFrame_raidgroup3", "TOPRIGHT", 0, 0}},  
    raidgroup5 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 5, 
        default_layout = {"TOPLEFT", "tufFrame_raidgroup1", "BOTTOMLEFT", 0, 0}},  
    raidgroup6 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 6, 
        default_layout = {"TOPLEFT", "tufFrame_raidgroup5", "TOPRIGHT", 0, 0}},  
    raidgroup7 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 7, 
        default_layout = {"TOPLEFT", "tufFrame_raidgroup6", "TOPRIGHT", 0, 0}},  
    raidgroup8 = {unit = "N/A", config = "raid", frame_type = "raidgroup", group = 8, 
        default_layout = {"TOPLEFT", "tufFrame_raidgroup7", "TOPRIGHT", 0, 0}} 
}

--[[
    Registers all frames with IFM
]]
function TuckUF.controlframe:RegisterWithIFM()
    for framename, config in pairs(unit_frames) do
        local frame = config.frame
        if frame and config.frame_type == "unit" then
            IFrameManager:Register(frame, ifm_iface, config.default_layout)
        elseif frame and config.frame_type == "raidgroup" then
            IFrameManager:Register(frame, ifm_raid_iface, config.default_layout)
        end
    end
end

--[[
    Resets all layouts to the default
]]
function TuckUF.controlframe:ResetAllLayouts()
    for framename, config in pairs(unit_frames) do
        local layout = config.default_layout
        local frame = config.frame
        if frame and layout then
            frame:ClearAllPoints()
            frame:SetPoint(layout[1], layout[2], layout[3], layout[4], layout[5])
        end
    end
end

--[[
    Initializes TuckUF Frames
]]
function TuckUF.controlframe:InitTUF()  
	-- register unit events for dispatch
	local unitEvents = {
			"UNIT_AURA", "UNIT_HEALTH", "UNIT_MANA", "UNIT_RAGE",
			"UNIT_FOCUS", "UNIT_ENERGY",
			"UNIT_LEVEL", "UNIT_NAME_UPDATE", "UNIT_PET",
			"PLAYER_COMBO_POINTS", "UNIT_DISPLAYPOWER"}
			
	for k,v in pairs(unitEvents) do
			TuckUFControlFrame:RegisterEvent(v)
	end
	
	-- register control events for frame managements
	TuckUFControlFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
	TuckUFControlFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
	TuckUFControlFrame:RegisterEvent("RAID_ROSTER_UPDATE")
	TuckUFControlFrame:RegisterEvent("PARTY_MEMBERS_CHANGED")
	TuckUFControlFrame:RegisterEvent("CHAT_MSG_ADDON")

	-- register onUpdate function
	TuckUFControlFrame:SetScript("OnUpdate", tuf_controlframe.OnUpdate)
    
    -- create all unit frames
    for framename, config in pairs(unit_frames) do
        if config.frame_type == "unit" then
            config.frame = tuf_unitframe:CreateFrame(framename, config.unit, config.config)
        elseif config.frame_type == "raidgroup" then
            config.frame = CreateFrame("Frame", "tufFrame_"..framename, UIParent)
            local width = tuf_unitframe:GetOption(config.config, "width") or 100
            local height = tuf_unitframe:GetOption(config.config, "height") or 20
            config.frame:SetWidth(width + 2 + 1)
            config.frame:SetHeight(6*(height + 2) + 1)
            config.frame.group = config.group
            
            local headerName = "tufRaidHeader"..config.group
            local header = CreateFrame("Frame", headerName, config.frame, "SecureRaidGroupHeaderTemplate")
            header.group = config.group
            header:SetAttribute("groupFilter",config.group)
            header:SetAttribute("yOffset",0)
            header:SetAttribute("template","TuckUFRaidUnitTemplate")
            header:SetAttribute("template","TuckUFRaidUnitTemplate")
            header.initialConfigFunction = tuf_controlframe.InitRaidFrame
            header:Show()
            header:SetPoint("TOPLEFT", config.frame, "TOPLEFT", 0, 0);
        end
    end
    
    -- initialize all unit frames
    tuf_controlframe:InitializeAllFrames()

    -- Register all frames with IFM
	tuf_controlframe:RegisterWithIFM()

	-- Combo Points
	CreateFrame("Frame", "tufComboFrame", UIParent, "TuckUFComboTemplate")
    tufComboFrameIcon:ClearAllPoints()
	tufComboFrameIcon:SetAllPoints(tufComboFrame)
    tufComboFrame:ClearAllPoints()
	tufComboFrame:SetPoint("TOPRIGHT", "tufFrame_target", "TOPLEFT", 0, 0)
    tufComboFrame:SetPoint("BOTTOMRIGHT", "tufFrame_target", "BOTTOMLEFT", 0, 0)
    tuf_controlframe:PLAYER_COMBO_POINTS()
    
    -- Assist Frame
    local focus_frame = tuf_controlframe.unitframes["focus"]
    if focus_frame ~= nil then
        CreateFrame("Frame", "tufAssistFrame", UIParent)
        tufAssistFrame:ClearAllPoints()
        tufAssistFrame:SetAllPoints(focus_frame)
        tufAssistFrame:SetBackdrop({bgFile = "Interface\\Addons\\TuckUnitFrames\\Images\\FrameBack", 
                            edgeFile = "Interface\\Addons\\TuckUnitFrames\\Images\\custom", 
                            tile = true, tileSize = 16, edgeSize = border_size, 
                            insets = { left = border_size, right = border_size, top = border_size, bottom = border_size }})
        tufAssistFrame:SetBackdropColor(0.0, 0.5, 1.0)
        tufAssistFrame:SetBackdropBorderColor(0.0, 0.5, 1.0)
        tufAssistFrame:CreateFontString("tufAssistFrameText", "OVERLAY", "TuckUFComboFont")
        tufAssistFrameText:ClearAllPoints()
        tufAssistFrameText:SetAllPoints(tufAssistFrame)
        tufAssistFrameText:SetText("class")
        tufAssistFrame:Hide()
    end
end

--[[
    Initializes all frames
]]
function TuckUF.controlframe:InitializeAllFrames()
    for unit, frame in pairs(tuf_controlframe.unitframes) do
        if frame then
            tuf_unitframe:OnInit(frame)
        end
    end
    
    if tufComboFrame then
        tufComboFrame:SetWidth(tufComboFrame:GetHeight())
    end
end

--[[
    Initializes a raid header (secure template stuff)
]]
function TuckUF.controlframe.InitRaidFrame(frame)
    local width = tuf_unitframe:GetOption("raid", "width") or 60
    local height = tuf_unitframe:GetOption("raid", "height") or 20
	frame:SetAttribute("initial-width", width)
	frame:SetAttribute("initial-height", height)

	tuf_unitframe:OnInit(frame)
end

--[[
    OnUpdate function, called at each frame
]]
function TuckUF.controlframe.OnUpdate(self, elapsed)

    -- time between updates, in seconds
    local update_delta = 0.1
    
	-- progress timer
	tuf_controlframe.timer = tuf_controlframe.timer - elapsed;		
	tuf_controlframe.elapsed = tuf_controlframe.elapsed + elapsed;	
	
	-- restrict frame updates to 10x per second
	if ( tuf_controlframe.timer <= 0 ) then
		tuf_controlframe.timer = update_delta
	
		-- update state
		tuf_network:OnUpdate(tuf_controlframe.elapsed)
		
		-- update frames
        for unit, config in pairs(unit_frames) do
            local frame = config.frame
            if frame and config.manual_update then
                tuf_unitframe:updateAll(frame)
            end
        end
        
        -- update assist frame
        tufAssistFrame:Hide()
        if not UnitExists("focus") then
            for i, unit in ipairs(tuf_controlframe.assist_units) do
                local target = unit.."target"
                if UnitExists(unit) and UnitExists(target) and UnitIsEnemy("player", target) and not UnitIsUnit(target, "target") then
                    local target_name = UnitClass(target) or UnitCreatureType(target) or UnitCreatureFamily(target) or "???"
                    tufAssistFrameText:SetText(target_name)
                    tufAssistFrame:Show()
                end
            end
        end 
   
	    -- store update time
		tuf_controlframe.lastupdate = time
		tuf_controlframe.elapsed = 0
	end
end

--[[
    Universal event handler
]]
function TuckUF.controlframe:OnControlEvent()
	tuf_controlframe[event]()
end

--[[
    Call after aura information from other players was received
]]
function TuckUF.controlframe:OnNetworkAura(unit_name)
    -- update all frames
    for unit, frame in pairs(tuf_controlframe.unitframes) do
        if frame then
            tuf_unitframe:updateAuras(frame)
        end
    end
end

--[[
    Call after aura information from other players was received
]]
function TuckUF.controlframe:OnNetworkComboPoint(unit_name)
    -- update all frames
    for unit, frame in pairs(tuf_controlframe.unitframes) do
        if frame then
            tuf_unitframe:updateMana(frame)
        end
    end
end



--[[
    Network traffic
]]
function TuckUF.controlframe:CHAT_MSG_ADDON()
	local prefix = arg1
	local msg = arg2
	local source = arg3
	local sender = arg4

    --TRACE("raw receive: "..prefix..": "..msg)
end

--[[
    Event: combo points changed
]]
function TuckUF.controlframe:PLAYER_COMBO_POINTS()
    -- broadcast CP info
    tuf_network:OnPlayerComboPoints()
    
	if GetComboPoints() > 0 then
		tufComboFrameCount:SetText(GetComboPoints())
		tufComboFrame:Show()
	else
		tufComboFrameCount:SetText("")
		tufComboFrame:Hide()
	end
end

--[[
    Event: unit aura changed
]]
function TuckUF.controlframe:UNIT_AURA()
    -- broadcast aura info
    tuf_network:OnUnitAura(arg1)
    
    -- update corresponding frame
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updateAuras(frame)
	end
end

--[[
    Event: unit pet changed
]]
function TuckUF.controlframe:UNIT_PET()   
	if (arg1=="player") and (tuf_controlframe.unitframes["pet"]) then
        local frame = tuf_controlframe.unitframes["pet"]
		tuf_unitframe:updateAll(frame)
	elseif tuf_controlframe.unitframes[arg1.."pet"] then
        local frame = tuf_controlframe.unitframes[arg1.."pet"]
		tuf_unitframe:updateAll(frame)
	end
end

--[[
    Event: unit HP changed
]]
function TuckUF.controlframe:UNIT_HEALTH()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updateHealth(frame)
	end
end

--[[
    Event: unit mana changed
]]
function TuckUF.controlframe:UNIT_MANA()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updatePowerType(frame)
		tuf_unitframe:updateMana(frame)
	end
end

--[[
    Event: 
]]
function TuckUF.controlframe:UNIT_RAGE()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updatePowerType(frame)
		tuf_unitframe:updateMana(frame)
	end
end

--[[
    Event: 
]]
function TuckUF.controlframe:UNIT_FOCUS()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updatePowerType(frame)
		tuf_unitframe:updateMana(frame)
	end
end

--[[
    Event: 
]]
function TuckUF.controlframe:UNIT_ENERGY()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updatePowerType(frame)
		tuf_unitframe:updateMana(frame)
	end
end

--[[
    Event: 
]]
function TuckUF.controlframe:UNIT_NAME_UPDATE()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updateNameText(frame)
	end
end

--[[
    Event: 
]]
function TuckUF.controlframe:UNIT_LEVEL()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updateClassText(frame)
	end
end

--[[
    Event: 
]]
function TuckUF.controlframe:UNIT_DISPLAYPOWER()
    local frame = tuf_controlframe.unitframes[arg1]
	if frame then
		tuf_unitframe:updatePowerType(frame)
	end
end

--[[
    Event: 
]]
function TuckUF.controlframe:RAID_TARGET_UPDATE()

end

--[[
    Event: 
]]
function TuckUF.controlframe:PLAYER_TARGET_CHANGED()
    local frame = tuf_controlframe.unitframes["target"]
    if frame then
        tuf_unitframe:updateAll(frame)
    end
end

--[[
    Event: 
]]
function TuckUF.controlframe:PLAYER_FOCUS_CHANGED()
    local frame = tuf_controlframe.unitframes["focus"]
    if frame then
        tuf_unitframe:updateAll(frame)
    end
end

--[[
    Event: raid composition changed
]]
function TuckUF.controlframe:RAID_ROSTER_UPDATE()
	for unit, frame in pairs(tuf_controlframe.unitframes) do
        if frame then
            tuf_unitframe:updateAll(frame)
        end
	end
end

--[[
    Event: party composition changed
]]
function TuckUF.controlframe:PARTY_MEMBERS_CHANGED()
	for i=1,4 do
        local frame = tuf_controlframe.unitframes["party"..i]
        if frame then
            tuf_unitframe:updateAll(frame)	
        end
        frame = tuf_controlframe.unitframes["partypet"..i]
        if frame then
            tuf_unitframe:updateAll(frame)	
        end
	end
end


